Module: Protocol::URL::Encoding::System
- Defined in:
- lib/protocol/url/encoding.rb
Overview
Maps individual URL path segments to and from local filesystem components.
Unlike generic URL decoding, this encoding rejects values which would turn one URL segment into multiple local path components.
Class Method Summary collapse
-
.escape(component) ⇒ Object
Encode one local filesystem component as one URL path segment.
-
.unescape(segment) ⇒ Object
Decode one URL path segment as one local filesystem component.